import "core-js/modules/es6.regexp.to-string";
import "core-js/modules/es7.array.includes";
import "core-js/modules/es6.string.includes";
import "core-js/modules/es6.regexp.match";
import "core-js/modules/es6.promise";
import _objectSpread from "@babel/runtime/helpers/objectSpread";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";

/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
import isString from 'lodash/isString';
import merge from 'lodash/merge';
import URL from 'url';
import qs from 'qs';
import pako from 'pako';

function isFormUrlEncoded(contentType) {
  return contentType && contentType.indexOf('x-www-form-urlencoded') >= 0;
}
/**
 * Creates options for a fetch call
 * @param {String} url The URL to fetch
 * @param {Object} fetchOptions fetch options
 * @param {String} qsOptions Options for serializing the request body using the qs package
 */


function createRequestOptions(url, fetchOptions, qsOptions) {
  var body = fetchOptions.body,
      _fetchOptions$headers = fetchOptions.headers,
      headers = _fetchOptions$headers === void 0 ? {} : _fetchOptions$headers,
      method = fetchOptions.method,
      options = _objectWithoutProperties(fetchOptions, ["body", "headers", "method"]);

  if (body) {
    method = method || 'POST'; // Only apply stringifying when body is not a string already

    if (!isString(body)) {
      if (isFormUrlEncoded(headers['content-type'])) {
        body = qs.stringify(body, qsOptions);
      } else {
        body = JSON.stringify(body);
        headers['content-type'] = 'application/json';
      }
    }

    headers['content-length'] = Buffer.byteLength(body);
  }

  var _URL$parse = URL.parse(url),
      hostname = _URL$parse.hostname,
      port = _URL$parse.port,
      path = _URL$parse.path;

  return _objectSpread({}, options, {
    method: method,
    body: body,
    hostname: hostname,
    port: port,
    path: path,
    headers: headers
  });
}
/**
 * The same as fetch, but automatically relays the cookies passed in from the browser to the upstream API.
 * @param {String} url The url to fetch
 * @param {Object} options Options for fetch
 * @param {Object} [options.redirect=manual] "manual", "follow", or "error"
 * @param {Object} [options.maxRedirects=20] The maximum number of redirects that fetch will follow before returning an error.  Defaults to 20.
 * @param {Object} [options.acceptInvalidCerts=false] Set to true to allow connections to sites with invalid SSL cers.
 * @param {String} qsOptions Options for serializing the request body using the qs package
 * @return {Promise}
 */


export function fetchWithCookies(url) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var qsOptions = arguments.length > 2 ? arguments[2] : undefined;
  var headers = {};

  if (env.cookie && env.shouldSendCookies !== false) {
    headers.cookie = env.cookie;
  }

  return fetch(url, merge(options, {
    credentials: 'include',
    headers: headers
  }), qsOptions);
}
/**
 * An implementation of the standard fetch API.  See https://developer.mozilla.org/en-US/docs/Web/API/Fetch_API for options
 *
 * This function is commonly used to fetch json data
 *
 * ```js
 * const data = await fetch('https://jsonplaceholder.typicode.com/todos/1')
 *   .then(res => res.json())
 * ```
 *
 * Or string data:
 *
 * ```js
 * const text = await fetch('https://jsonplaceholder.typicode.com/todos/1')
 *   .then(res => res.text())
 * ```
 *
 * You can also use it to fetch binary data:
 *
 * ```js
 * const buffer = await fetch('https://www.w3.org/WAI/ER/tests/xhtml/testfiles/resources/pdf/dummy.pdf')
 *   .then(res => res.arrayBuffer())
 * ```
 *
 * @param {String} url The URL to fetch
 * @param {Object} options Options for fetch. In addition to the standard fetch options, you can also specify:
 * @param {Object} [options.redirect=manual] "manual", "follow", or "error"
 * @param {Object} [options.maxRedirects=20] The maximum number of redirects that fetch will follow before returning an error.  Defaults to 20.
 * @param {Object} [options.acceptInvalidCerts=false] Set to true to allow connections to sites with invalid SSL cers.
 * @param {Object} qsOptions Options for serializing the request body using the qs package
 * @return {Promise}
 */

export default function fetch(url) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var qsOptions = arguments.length > 2 ? arguments[2] : undefined;
  return new Promise(function (resolve, reject) {
    var protocol = url.match(/^https/) ? global.https : global.http;

    var _createRequestOptions = createRequestOptions(url, options, qsOptions),
        body = _createRequestOptions.body,
        requestOptions = _objectWithoutProperties(_createRequestOptions, ["body"]); // use native node buffer, not webpack's shim, so it can be passed with response.send() on Moov XDN


    var Buffer = global.Buffer; // Moov should inject this via the Server component

    var req = protocol.request(requestOptions, function (response) {
      var data = [];
      relaySetCookies(response, requestOptions.hostname);
      response.on('data', function (chunk) {
        return data.push(chunk);
      });
      response.on('end', function () {
        data = Buffer.concat(data);

        if ([301, 302].includes(response.statusCode) && response.headers.location) {
          // redirects
          var _options$maxRedirects = options.maxRedirects,
              maxRedirects = _options$maxRedirects === void 0 ? 20 : _options$maxRedirects;

          if (options.redirect === 'follow') {
            if (maxRedirects > 0) {
              var redirectURL = URL.resolve(url, response.headers.location);
              return resolve(fetch(redirectURL, _objectSpread({}, options, {
                maxRedirects: maxRedirects - 1,
                redirected: true
              })));
            } else {
              return reject(new Error('The maximum number of redirects has been reached while using fetch.'));
            }
          } else if (options.redirect === 'error') {
            return reject(new Error("fetch received a redirect response status ".concat(response.statusCode, " and options.redirect was set to \"error\".")));
          } else {
            var redirectData = {
              redirect: response.headers.location
            };
            return resolve({
              redirected: true,
              url: response.headers.location,
              status: response.statusCode,
              ok: true,
              headers: response.headers,
              text: function text() {
                return Promise.resolve(JSON.stringify(redirectData));
              },
              json: function json() {
                return Promise.resolve(redirectData);
              }
            });
          }
        } else {
          // all other success and error response codes
          var ok = response.statusCode >= 200 && response.statusCode <= 299;
          var result = {
            redirected: options.redirected || false,
            url: url,
            status: response.statusCode,
            statusText: response.statusText,
            ok: ok,
            headers: response.headers,
            arrayBuffer: function arrayBuffer() {
              return Promise.resolve(data);
            },
            text: function text() {
              return Promise.resolve(extractString(response, data));
            },
            json: function json() {
              return Promise.resolve(JSON.parse(extractString(response, data)));
            }
          };

          if (!ok) {
            var error = new Error("".concat(response.statusCode, ": ").concat(data.toString('utf8')));
            error.response = result;
            reject(error);
            return;
          } // Recreating simple API similar to Fetch
          // https://developer.mozilla.org/en-US/docs/Web/API/Fetch_API


          resolve(result);
        }
      });
    });
    req.on('error', function (err) {
      reject(err);
    }); // do not break request when we have no data

    if (body) {
      req.write(body);
    }

    req.end();
  });
}

function extractString(response, data) {
  if (response.headers['content-encoding'] === 'gzip') {
    return pako.inflate(data, {
      to: 'string'
    });
  } else {
    return data.toString('utf8');
  }
}
/**
 * Adds a set-cookie header to env.MUR_SET_COOKIES so we can relay it to the browser in Response.send
 * This is the mechanism by which we propagate sessions created in MUR requests.
 * @param {Object} request The request object
 * @param {String} domain The request hostname
 */


function relaySetCookies(request, domain) {
  var cookie = request.headers['set-cookie'];

  if (cookie) {
    var cookies = env.MUR_SET_COOKIES || {};
    cookies[domain] = (cookies[domain] || []).concat(cookie);
    fns.export('MUR_SET_COOKIES', cookies);
  }
}
//# sourceMappingURL=fetch.js.map