import _extends from "@babel/runtime/helpers/extends";
import "core-js/modules/es6.object.assign";
import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import "core-js/modules/es6.regexp.to-string";
import "core-js/modules/es6.string.iterator";
import "core-js/modules/es6.map";
import "core-js/modules/es6.regexp.match";
import "core-js/modules/es6.regexp.split";
import "core-js/modules/web.dom.iterable";
import "core-js/modules/es6.array.iterator";
import "core-js/modules/es6.object.keys";
import "core-js/modules/es6.string.ends-with";
import "core-js/modules/es6.array.find";
import "core-js/modules/es6.regexp.replace";

/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
import React from 'react';
import ReactDOM from 'react-dom';
import { renderToString } from 'react-dom/server';
import { Provider } from 'mobx-react';
import { SheetsRegistry } from 'react-jss/lib/jss';
import JssProvider from 'react-jss/lib/JssProvider';
import { create } from 'jss';
import { jssPreset } from '@material-ui/core/styles';
import jssNested from 'jss-nested';
import { MuiThemeProvider } from '@material-ui/core/styles';
import createGenerateClassName from './utils/createGenerateClassName'; // Convert mobx-state-tree model to json and escape <\ in a closing script tag to avoid untimely script closing.

var getSanitizedModelJson = function getSanitizedModelJson(state) {
  return sanitizeJsonForScript(state.toJSON());
}; // Escape <\ in a closing script tag to avoid untimely script closing


var sanitizeJsonForScript = function sanitizeJsonForScript(state) {
  return JSON.stringify(state || {}).replace(/<\/script/gi, '<\\/script');
};
/**
 * Get javascript asset filename by chunk name
 * @private
 * @param  {Object} stats Webpack generated stats
 * @param  {String} chunk Chunk name
 * @return {String}       Asset filename
 */


export function getJS(stats, chunk) {
  var assets = stats.assetsByChunkName[chunk];

  if (!assets) {
    return null;
  } else if (Array.isArray(assets)) {
    return assets.find(function (file) {
      return file.endsWith('.js');
    });
  } else {
    return assets;
  }
}
/**
 * Gets all source paths for the given chunk
 * @private
 * @param  {Object} stats Webpack generated stats
 * @param  {String} chunk Chunk name
 * @return {String[]}
 */

function getSources(stats, chunk) {
  return Object.keys(stats.assetsByChunkName).filter(function (key) {
    // render any that have `chunk` in the name, split by ~
    // when webpack sees two or more chunks that share some vendor libs, it can decide
    // to create a separate chunk just for the libs those two chunks share.
    // for example: vendors~component-Component~guide-Guide
    return key.split('~').some(function (page) {
      return page === chunk;
    });
  }).map(function (key) {
    var assets = stats.assetsByChunkName[key];

    if (Array.isArray(assets)) {
      // we'll get an array when building with source maps, in that case pick the JS file, not the map
      return stats.assetsByChunkName[key].find(function (file) {
        return !file.match(/\.map$/);
      });
    } else {
      // otherwise the value will be the JS file
      return assets;
    }
  });
}
/**
 * Render HTML of given component
 * @private
 * @param  {options} options
 * @param  {React.Component} options.component Component to be rendered
 * @param  {Object} options.providers          Data providers
 * @param  {Object} options.registry           JSS Sheets Registry
 * @param  {Object} options.theme              MUI Theme
 * @param  {Object} options.cssPrefix          A prefix to apply to css classes
 * @return {String}                            HTML
 */


export function renderHtml(_ref) {
  var component = _ref.component,
      providers = _ref.providers,
      registry = _ref.registry,
      theme = _ref.theme,
      _ref$cssPrefix = _ref.cssPrefix,
      cssPrefix = _ref$cssPrefix === void 0 ? 'jss' : _ref$cssPrefix;
  return renderToString(React.createElement(Provider, providers, React.createElement(JssProvider, {
    classNamePrefix: cssPrefix,
    registry: registry,
    generateClassName: createGenerateClassName()
  }, React.createElement(MuiThemeProvider, {
    theme: theme,
    sheetsManager: new Map()
  }, component))));
}
/**
 * Renders initial state for client side hydration
 * @private
 * @param  {options} options                      Model instance
 * @param  {Object} options.state                 The initial app state needed for hydration
 * @param  {Object} options.routeData             The data that resulted from the route being run.  This is needed for creating a cache
 *                                                entry for the equivalend .json request if the user navigates back to this page.
 * @param  {Boolean} options.defer                Should defer execution
 * @param  {String} options.initialStateProperty  Property name on window object
 * @return {String}                               Script HTML
 */

export function renderInitialStateScript(_ref2) {
  var state = _ref2.state,
      routeData = _ref2.routeData,
      defer = _ref2.defer,
      _ref2$initialStatePro = _ref2.initialStateProperty,
      initialStateProperty = _ref2$initialStatePro === void 0 ? 'initialState' : _ref2$initialStatePro,
      _ref2$initialRouteDat = _ref2.initialRouteDataProperty,
      initialRouteDataProperty = _ref2$initialRouteDat === void 0 ? 'initialRouteData' : _ref2$initialRouteDat;
  return "<script type=\"text/javascript\" ".concat(defer ? 'defer' : '', ">\n    window.").concat(initialStateProperty, "=Object.freeze(").concat(getSanitizedModelJson(state), ")\n    window.").concat(initialRouteDataProperty, "=Object.freeze(").concat(sanitizeJsonForScript(routeData), ")\n\t</script>");
}
/**
 * Renders script for a specified chunk
 * @private
 * @param  {String} src             Source of script
 * @param  {Boolean} defer          Should defer execution
 * @return {String}                 Script HTML
 */

export function renderScript(src, defer) {
  return "<script type=\"text/javascript\" ".concat(defer ? 'defer' : '', " src=\"").concat(src, "\"></script>");
}
/**
 * Extracts scripts from sources in chunk
 * @private
 * @param {Object} options
 * @param {Object} options.stats    Webpack generated stats
 * @param {String} options.chunk     Chunk name
 */

export function getScripts(_ref3) {
  var stats = _ref3.stats,
      chunk = _ref3.chunk;
  var assetPathBase = process.env.ASSET_PATH_BASE || '';
  return getSources(stats, chunk).map(function (src) {
    return "".concat(assetPathBase, "/pwa/").concat(src);
  });
}
/**
 * Renders a link prefetch header value
 * @private
 * @param {String} src Source of script
 */

export function renderPreloadHeader(src) {
  return "<".concat(src, ">; rel=preload; as=script");
}
/**
 * Renders extracted CSS from Sheets Registry
 * @private
 * @param  {options} options
 * @param  {Object} options.registry  JSS Sheets Registry
 * @param  {String} options.id        ID for style tag
 * @return {String}                   Style HTML
 */

export function renderStyle(_ref4) {
  var registry = _ref4.registry,
      _ref4$id = _ref4.id,
      id = _ref4$id === void 0 ? 'ssr-css' : _ref4$id;
  return "<style id=\"".concat(id, "\">").concat(registry.toString(), "</style>");
}
/**
 * Helper for extracting location object from Adapt environment
 * @private
 * @param  {Object} env Adapt Environment
 * @return {Object}     Location Object
 */

function getLocation(env) {
  var _env$path$split = env.path.split('?'),
      _env$path$split2 = _slicedToArray(_env$path$split, 2),
      pathname = _env$path$split2[0],
      search = _env$path$split2[1];

  return {
    protocol: env.secure ? 'https' : 'http',
    hostname: env.host,
    pathname: pathname,
    search: search || ''
  };
}
/**
 * Renders a component on the server.
 * @private
 * @param  {options} options
 * @param  {React.Component} options.component    Component to be rendered
 * @param  {Object} options.state                 Model instance
 * @param  {Object} options.theme                 MUI Theme
 * @param  {Object} options.stats                 Webpack generated stats
 * @param  {String} options.clientChunk           Chunk name for hydration script
 * @param  {String} options.initialStateProperty  Optional window property name for initial state
 * @param  {Boolean} options.injectAssets          Defaults to true.  Set this to false to prevent scripts and css from automatically being injected into the document.
 * @param  {String} options.cssPrefix            A prefix to apply to css class names
 * @return {Object}                               Components for SSR
 */


export function render(_ref5) {
  var component = _ref5.component,
      state = _ref5.state,
      theme = _ref5.theme,
      stats = _ref5.stats,
      clientChunk = _ref5.clientChunk,
      initialStateProperty = _ref5.initialStateProperty,
      _ref5$injectAssets = _ref5.injectAssets,
      injectAssets = _ref5$injectAssets === void 0 ? true : _ref5$injectAssets,
      cssPrefix = _ref5.cssPrefix;
  var registry = new SheetsRegistry();
  state.applyState({
    location: getLocation(env)
  }, 'REPLACE');
  var html = renderHtml({
    component: component,
    providers: {
      app: state
    },
    registry: registry,
    theme: theme,
    cssPrefix: cssPrefix
  });
  var result = {
    html: html,
    style: renderStyle({
      registry: registry
    }),
    initialStateScript: renderInitialStateScript({
      state: state,
      defer: false,
      initialStateProperty: initialStateProperty
    }),
    componentScript: getScripts({
      stats: stats,
      chunk: clientChunk
    }).map(renderScript)
  };

  if (injectAssets) {
    var _$body;

    $head.append(result.style);
    $body.append(result.initialStateScript);

    (_$body = $body).append.apply(_$body, _toConsumableArray(result.componentScript));
  }

  return result;
}
/**
 * Hydrates React component
 * @private
 * @param  {options} options
 * @param  {React.Component} options.component    Component to be rendered
 * @param  {Model} options.model                  MobX Model
 * @param  {Object} options.theme                 MUI Theme
 * @param  {String} options.target                Selector for rendering target
 * @param  {String} options.providerProps         Props to add the to <Provider/> element
 * @param  {String} options.initialStateProperty  Optional window property name for initial state
 * @return {Object} The app state
 */

export function hydrate(_ref6) {
  var component = _ref6.component,
      model = _ref6.model,
      theme = _ref6.theme,
      target = _ref6.target,
      _ref6$providerProps = _ref6.providerProps,
      providerProps = _ref6$providerProps === void 0 ? {} : _ref6$providerProps,
      _ref6$initialStatePro = _ref6.initialStateProperty,
      initialStateProperty = _ref6$initialStatePro === void 0 ? 'initialState' : _ref6$initialStatePro,
      _ref6$cssPrefix = _ref6.cssPrefix,
      cssPrefix = _ref6$cssPrefix === void 0 ? 'jss' : _ref6$cssPrefix;
  var generateClassName = createGenerateClassName();
  var state = model.create(window[initialStateProperty] || {});
  var jss = create(jssPreset(), jssNested());
  window.moov = window.moov || {};
  Object.assign(window.moov, {
    state: state,
    timing: {}
  }, providerProps); // skip jss insertion if we are hydrating SSR cached by the service worker because it has already been mounted

  if (!document.body.hasAttribute('data-jss-inserted')) {
    var styleNode = document.createComment('jss-insertion-point');
    document.head.insertBefore(styleNode, document.head.firstChild);
    jss.options.insertionPoint = 'jss-insertion-point';
    document.body.setAttribute('data-jss-inserted', 'on');
  }

  ReactDOM.hydrate(React.createElement(Provider, _extends({
    app: state
  }, providerProps), React.createElement(JssProvider, {
    classNamePrefix: cssPrefix,
    jss: jss,
    generateClassName: generateClassName
  }, React.createElement(MuiThemeProvider, {
    theme: theme
  }, component))), target, removeSSRStyles);
  return state;
}
/**
 * Removes the style tag rendered on the server so that components are styled correctly
 * after hydration.
 * @private
 */

function removeSSRStyles() {
  var jssStyles = document.getElementById('ssr-css');

  if (jssStyles && jssStyles.parentNode) {
    jssStyles.parentNode.removeChild(jssStyles);
  }
}
//# sourceMappingURL=renderers.js.map